"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
// import { ux } from 'cli-ux';
const fs = require("fs-extra");
const request = require("request-promise-native");
const describes = {};
let conn;
const writeJSONOptions = {
    spaces: 2
};
class CDCPrep extends command_1.SfdxCommand {
    async run() {
        // get each object
        conn = await this.org.getConnection();
        const recordsFolder = `${this.flags.dir}/cdc/records`;
        const objectList = await fs.readdir(recordsFolder);
        for (const objectType of objectList) {
            this.ux.log(`prepping object ${objectType}`);
            // build an array from the objects
            let outputArray = [];
            let cdcItemList = await fs.readdir(`${recordsFolder}/${objectType}`);
            // sort by replayId
            cdcItemList = cdcItemList.sort((a, b) => parseInt(a, 10) - parseInt(b, 10));
            for (const cdcItem of cdcItemList) {
                const cdc = await fs.readJSON(`${recordsFolder}/${objectType}/${cdcItem}`);
                outputArray = await cdcToArray(outputArray, cdc);
            }
            await fs.outputJSON(`${this.flags.dir}/cdc/originalIds/${objectType}.json`, outputArray, writeJSONOptions);
        }
    }
}
CDCPrep.examples = [
// 'sfdx shane:cdc:stream // get all the change events',
// 'sfdx shane:cdc:stream -o Account // get all the change events on a single object',
// 'sfdx shane:cdc:stream -d myDir // stream change events to myDir/cdc, organized into folders by object api type'
];
CDCPrep.flagsConfig = {
    dir: command_1.flags.directory({ char: 'd', required: true, description: 'folder to upload, containing /cdc/records' })
};
CDCPrep.requiresProject = true;
CDCPrep.requiresUsername = true;
exports.default = CDCPrep;
// converts a cdc item to a insertable record
const cdcToArray = async (outputArray, cdc) => {
    let modifiedArray = Array.from(outputArray);
    const cdcRecordIds = cdc.payload.ChangeEventHeader.recordIds;
    // const entityType = cdc.payload.ChangeEventHeader.entityType;
    const changeType = cdc.payload.ChangeEventHeader.changeType;
    if (changeType === 'CREATE') {
        const newRecord = await cdcToRecord(cdc);
        modifiedArray.push(newRecord);
    }
    else if (changeType === 'DELETE') {
        modifiedArray = modifiedArray.filter(record => !cdcRecordIds.includes(record.Id));
    }
    else if (changeType === 'UPDATE') {
        for (const recordId of cdcRecordIds) {
            const recordKey = modifiedArray.findIndex(record => record.Id === recordId);
            const oldRecord = modifiedArray[recordKey];
            const newRecord = await cdcToRecord(cdc);
            modifiedArray[recordKey] = Object.assign({}, oldRecord, newRecord);
        }
    }
    else if (changeType === 'UNDELETE') {
        // NOT HANDLED
    }
    return modifiedArray;
};
const cdcToRecord = async (cdc) => {
    const recordOutput = {
        Id: cdc.payload.ChangeEventHeader.recordIds[0]
    };
    const entityType = cdc.payload.ChangeEventHeader.entityName;
    const describe = await getDescribe(entityType);
    const createableFieldsApiNames = Object.keys(describe.fields)
        .map(key => describe.fields[key])
        .filter(field => field.createable)
        .map(field => field.apiName);
    Object.keys(cdc.payload).forEach(key => {
        if (createableFieldsApiNames.includes(key)) {
            recordOutput[key] = cdc.payload[key];
        }
    });
    return recordOutput;
};
const getDescribe = async (entityType) => {
    // get and save for future use
    if (!describes[entityType]) {
        const apiVersion = await conn.retrieveMaxApiVersion();
        const uri = `${conn.instanceUrl}services/data/v${apiVersion}/ui-api/object-info/${entityType}`;
        const result = await request({
            method: 'get',
            uri,
            headers: {
                Authorization: `Bearer ${conn.accessToken}`
            },
            json: true
        });
        // save for future use
        describes[entityType] = result;
    }
    return describes[entityType];
};
//# sourceMappingURL=prep.js.map