"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable no-var-requires */
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const request = require("request-promise-native");
class Activation extends command_1.SfdxCommand {
    async run() {
        // hardcoded because I've been burned on this one before
        const version = '44.0';
        const conn = this.org.getConnection();
        const auth = {
            bearer: conn.accessToken
        };
        const options = {
            resetInstancesOnActivation: Boolean(this.flags.reset)
        };
        // first, get the orchestration Id
        const orchestrations = JSON.parse(await request.get(`${conn.instanceUrl}/services/data/v${version}/iot/orchestrations`, {
            auth
        })).orchestrations;
        // console.log(orchestrations);
        // find the matching orchestration by name
        const thisOrch = orchestrations.find(i => {
            return i.name === this.flags.name;
        });
        // didn't find a match.  Tell the user what's in there.
        if (!thisOrch) {
            const orchNames = orchestrations.map(x => x.name);
            this.ux.error(chalk_1.default.red(`No orchestration found matching that name.  Orchestrations found: ${orchNames.join(', ')}`));
            return;
        }
        else {
            // this.ux.logJson(thisOrch);
            this.ux.log(`found orchestration with id ${thisOrch.id}.  Will activate its context ${thisOrch.orchestrationContext.label}`);
        }
        const contextActivationResult = await request.post({
            url: `${conn.instanceUrl}/services/data/v${version}/iot/contexts/${thisOrch.orchestrationContext.id}/activations`,
            body: {},
            auth,
            json: true
        });
        this.ux.log(contextActivationResult);
        // wait for a sign that the context is active
        let contextActive = false;
        do {
            const context = JSON.parse(await request.get(`${conn.instanceUrl}/services/data/v${version}/iot/contexts/${thisOrch.orchestrationContext.id}`, {
                auth
            }));
            // this.ux.logJson(context);
            if (context.activationStatus === 'Active') {
                contextActive = true;
            }
        } while (!contextActive);
        if (this.flags.deactivate) {
            const deactivationResult = await request.patch({
                url: `${conn.instanceUrl}/services/data/v42.0/iot/orchestrations/${thisOrch.id}/activations/last`,
                body: {
                    status: 'Inactive',
                    options
                },
                auth,
                json: true
            });
            this.ux.log(chalk_1.default.green('Orchestration deactivating.'));
            return deactivationResult;
        }
        else {
            const activationResult = await request.post({
                url: `${conn.instanceUrl}/services/data/v42.0/iot/orchestrations/${thisOrch.id}/activations`,
                body: {
                    options
                },
                auth,
                json: true
            });
            this.ux.log(chalk_1.default.green('Orchestration activating.'));
            // wait for a sign that the context is active
            let orchActive = false;
            do {
                const newOrch = JSON.parse(await request.get(`${conn.instanceUrl}/services/data/v${version}/iot/orchestrations/${thisOrch.id}`, {
                    auth
                }));
                // this.ux.logJson(newOrch);
                if (newOrch.status === 'Active') {
                    orchActive = true;
                    this.ux.log(chalk_1.default.green('Orchestration is active.'));
                    return newOrch;
                }
            } while (!orchActive);
            return activationResult;
        }
    }
}
Activation.description = 'Activate an iot orchestration by name';
Activation.examples = [
    `sfdx shane:iot:activate -n orchName -r
// activates the orchestration, including the context if necessary, optionally resetting all the instances
`,
    `sfdx shane:iot:activate -n orchName -d
// deactivates the orchestration, without resetting all the instances
`
];
Activation.flagsConfig = {
    name: command_1.flags.string({ char: 'n', required: true, description: 'API name of the orchestration' }),
    reset: command_1.flags.boolean({ char: 'r', description: 'reset all instances of the orchestration' }),
    deactivate: command_1.flags.boolean({ char: 'd', description: 'deactivate the orchestration' })
};
Activation.requiresUsername = true;
Activation.requiresProject = false;
exports.default = Activation;
//# sourceMappingURL=activation.js.map