"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const fs = require("fs-extra");
const jsToXml = require("js2xmlparser");
const sw = require("stopword");
const getExisting_1 = require("../../../shared/getExisting");
const options = require("../../../shared/js2xmlStandardOptions");
const setupArray_1 = require("../../../shared/setupArray");
class LabelAdd extends command_1.SfdxCommand {
    async run() {
        const labelsFolder = `${this.flags.target}/labels`;
        const targetFilename = `${labelsFolder}/${this.flags.bundle}.labels-meta.xml`;
        await fs.ensureDir(labelsFolder);
        let existing = await getExisting_1.getExisting(targetFilename, 'CustomLabels', {
            '@': {
                xmlns: 'http://soap.sforce.com/2006/04/metadata'
            },
            labels: []
        });
        existing = setupArray_1.setupArray(existing, 'labels');
        const newLabel = {
            fullName: this.flags.name || sw.removeStopwords(this.flags.text.split(' ')).join(' ').replace(/[^a-zA-Z0-9]/g, '').substring(0, 80),
            shortDescription: this.flags.description || sw.removeStopwords(this.flags.text.split(' ')).join(' ').replace(/[^a-zA-Z0-9]/g, '').substring(0, 80),
            language: this.flags.language,
            protected: this.flags.protected || false,
            value: this.flags.text
        };
        if (this.flags.categories) {
            newLabel.categories = this.flags.categories.join(',');
        }
        // console.log(existing);
        // verify label doesn't already exist
        if (existing.labels.filter(label => label.fullName === newLabel.fullName).length > 0) {
            throw new Error(`A label with the fullName ${newLabel.fullName} already exists`);
        }
        existing.labels.push(newLabel);
        existing = await getExisting_1.fixExistingDollarSign(existing);
        // read the custom labels into json
        // build the new label
        // write it back
        const xml = jsToXml.parse('CustomLabels', existing, options.js2xmlStandardOptions);
        fs.writeFileSync(targetFilename, xml);
        this.ux.log(chalk_1.default.green(`Added ${newLabel.fullName} to ${targetFilename} in local source`));
        return existing;
    }
}
LabelAdd.description = 'create a remote site setting in the local source.  Push it when you\'re done';
LabelAdd.examples = [
    `sfdx shane:label:add -t "This is some Text"
// create a custom label with the displayed text and all the defaults
`
];
LabelAdd.flagsConfig = {
    text: command_1.flags.string({ required: true, char: 't', description: 'the text you want to turn into a label' }),
    bundle: command_1.flags.string({ description: 'label bundle when you want to organize them more', default: 'CustomLabels' }),
    name: command_1.flags.string({ char: 'n', description: 'api name for your label' }),
    description: command_1.flags.string({ char: 'd', description: 'description for your label' }),
    protected: command_1.flags.boolean({ description: 'mark as protected (packaged, subscribers cannot change the label' }),
    categories: command_1.flags.array({ description: 'categories to add to your custom label' }),
    language: command_1.flags.string({ char: 'l', description: 'language code', default: 'en_US' }),
    target: command_1.flags.directory({ char: 't', default: 'force-app/main/default', description: 'where to create the folder (if it doesn\'t exist already) and file...defaults to force-app/main/default' })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
LabelAdd.requiresProject = true;
exports.default = LabelAdd;
//# sourceMappingURL=add.js.map