"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const cli_ux_1 = require("cli-ux");
const fs = require("fs-extra");
const jsToXml = require("js2xmlparser");
const options = require("../../../shared/js2xmlStandardOptions");
const typeDefinitions = [
    { type: 'custom',
        forbidden: ['highvolume'],
        specificOptions: ['activities', 'search', 'feeds'],
        specificRequired: ['nametype', 'namefieldlabel'],
        ending: '__c'
    },
    {
        type: 'big',
        forbidden: ['highvolume', 'activities', 'sharing', 'search', 'feeds'],
        specificOptions: [],
        ending: '__b'
    },
    {
        type: 'event',
        forbidden: ['activities', 'sharing', 'search', 'feeds'],
        specificOptions: [],
        ending: '__e'
    }
];
class ObjectCreate extends command_1.SfdxCommand {
    constructor() {
        super(...arguments);
        this.validate = typename => {
            const found = typeDefinitions.find(t => t.type === typename);
            if (!found) {
                throw new Error(`invalid type specified (${typename})`);
            }
            found.forbidden.forEach(att => {
                if (this.flags[att]) {
                    throw new Error(`${att} is not valid for ${typename}`);
                }
            });
            return true;
        };
    }
    async run() {
        const outputJSON = {
            '@': {
                xmlns: 'http://soap.sforce.com/2006/04/metadata'
            },
            deploymentStatus: 'Deployed',
            label: '',
            pluralLabel: ''
        };
        // remove trailing slash if someone entered it
        if (this.flags.directory.endsWith('/')) {
            this.flags.directory = this.flags.directory.substring(0, this.flags.directory.length - 1);
        }
        if (!this.flags.type) {
            this.flags.type = await cli_ux_1.default.prompt(`Object type [${typeDefinitions.map(td => td.type)}]`);
        }
        if (!this.flags.label) {
            this.flags.label = await cli_ux_1.default.prompt('Label for the object?');
        }
        if (!this.flags.plural) {
            this.flags.plural = await cli_ux_1.default.prompt('Plural label for the object?', { default: `${this.flags.label}s` });
        }
        if (!this.flags.api) {
            let suffix = '__c';
            if (this.flags.type === 'big') {
                suffix = '__b';
            }
            else if (this.flags.type === 'event') {
                suffix = '__e';
            }
            this.flags.api = await cli_ux_1.default.prompt(`API name (ends with ${suffix}) ?`, { default: `${this.flags.label.replace(/ /g, '_')}${suffix}` });
        }
        // checks and throws an error if types and params don't mix
        this.validate(this.flags.type);
        if (this.flags.type === 'big') {
            outputJSON.indexes = {
                fullName: `${this.flags.api.replace('__b', '')}Index`,
                label: `${this.flags.label} Index`,
                fields: []
            };
        }
        outputJSON.label = this.flags.label;
        outputJSON.pluralLabel = this.flags.plural;
        // optional attributes for all types
        if (this.flags.description) {
            outputJSON.description = this.flags.description;
        }
        else if (this.flags.interactive) {
            outputJSON.description = await cli_ux_1.default.prompt('description?  Be nice to your future self!', { required: false });
        }
        // type specific attributes
        if (this.flags.type === 'event') {
            if (this.flags.interactive && !this.flags.highvolume) {
                this.flags.highvolume = await cli_ux_1.default.confirm('High Volume (y/n)');
            }
            outputJSON.eventType = this.flags.highvolume ? 'HighVolume' : 'StandardVolume';
        }
        // type specific attributes
        if (this.flags.type === 'custom') {
            if (this.flags.interactive && !this.flags.nametype) {
                this.flags.nametype = await cli_ux_1.default.prompt('AutoNumber or Text?', { default: 'Text' });
            }
            if (this.flags.interactive && !this.flags.search) {
                this.flags.search = await cli_ux_1.default.confirm('enable Search? (y/n)');
            }
            if (this.flags.interactive && !this.flags.feeds) {
                this.flags.feeds = await cli_ux_1.default.confirm('enable feeds? (y/n)');
            }
            if (this.flags.interactive && !this.flags.reports) {
                this.flags.reports = await cli_ux_1.default.confirm('enable reports? (y/n)');
            }
            if (this.flags.interactive && !this.flags.history) {
                this.flags.history = await cli_ux_1.default.confirm('enable history? (y/n)');
            }
            if (this.flags.interactive && !this.flags.activities) {
                this.flags.activities = await cli_ux_1.default.confirm('enable activities? (y/n)');
            }
            if (this.flags.interactive && !this.flags.namefieldlabel) {
                this.flags.namefieldlabel = await cli_ux_1.default.prompt('What do you want to call the name field?', { default: `${this.flags.label} Name` });
            }
            if (this.flags.interactive && !this.flags.sharing) {
                this.flags.sharing = await cli_ux_1.default.prompt('Sharing model? [ReadWrite, Read, Private]', { default: 'ReadWrite' });
            }
            if (this.flags.interactive && !this.flags.enterprise) {
                this.flags.enterprise = await cli_ux_1.default.confirm('Enable bulk API, sharing, and streaming API? (y/n)');
            }
            if (this.flags.nametype === 'AutoNumber' && !this.flags.autonumberformat) {
                this.flags.autonumberformat = await cli_ux_1.default.prompt('Display format', { default: `${this.flags.label}-{0}` });
            }
            outputJSON.nameField = {
                type: this.flags.nametype,
                label: this.flags.namefieldlabel
            };
            outputJSON.sharingModel = this.flags.sharingmodel;
            if (this.flags.nametype === 'AutoNumber' && this.flags.autonumberformat) {
                outputJSON.nameField.displayFormat = this.flags.autonumberformat;
            }
            if (this.flags.enterprise) {
                outputJSON.enableSharing = true;
                outputJSON.enableBulkApi = true;
                outputJSON.enableStreamingApi = true;
            }
            if (this.flags.reports) {
                outputJSON.enableReports = true;
            }
            if (this.flags.history) {
                outputJSON.enableHistory = true;
            }
            if (this.flags.activities) {
                outputJSON.enableActivities = true;
            }
            if (this.flags.feeds) {
                outputJSON.enableFeeds = true;
            }
            if (this.flags.search) {
                outputJSON.enableSearch = true;
            }
        }
        const objectsPath = `${this.flags.directory}/objects`;
        const thisObjectFolder = `${this.flags.directory}/objects/${this.flags.api}`;
        const metaFileLocation = `${this.flags.directory}/objects/${this.flags.api}/${this.flags.api}.object-meta.xml`;
        fs.ensureDirSync(objectsPath);
        if (!fs.existsSync(thisObjectFolder)) {
            fs.mkdirSync(thisObjectFolder);
        }
        else {
            this.ux.error(`Object already exists: ${thisObjectFolder}`);
            return;
        }
        fs.ensureDirSync(`${objectsPath}/${this.flags.api}/fields`);
        const xml = jsToXml.parse('CustomObject', outputJSON, options.js2xmlStandardOptions);
        fs.writeFileSync(metaFileLocation, xml);
        this.ux.log(`Created ${chalk_1.default.green(thisObjectFolder)}.  Add fields with ${chalk_1.default.cyan(`sfdx shane:object:field -o ${this.flags.api}`)}.`);
    }
}
ObjectCreate.description = 'create an object in local source.  Only __c (limited support), __b (big objects) and events __e are currently supported';
ObjectCreate.examples = [
    `sfdx shane:object:create
// without any params, the cli is going to ask you questions to generate your object interactively
`,
    `sfdx shane:object:create --label "Platypus" --plural "Platypi" --api Platypus__b --directory /my/project/path
// label, plural, api name specified so the tool doesn't have to ask you about them.  Creates in a non-default path
`,
    `sfdx shane:object:create --label "Platypus" --plural "Platypi" --api Platypus__b --directory /my/project/path
// label, plural, api name specified so the tool doesn't have to ask you about them.  Creates in a non-default path
`,
    `sfdx shane:object:create --label "Signal" --plural "Signals" --api Signal__e
// create a platform event
`
];
ObjectCreate.flagsConfig = {
    type: command_1.flags.string({ char: 't', description: 'type of object', options: typeDefinitions.map(td => td.type) }),
    // attributes on all types
    label: command_1.flags.string({ char: 'l', description: 'label for the UI' }),
    api: command_1.flags.string({ char: 'a', description: 'api name.  Ends with one of the supported types: [__b, __e]' }),
    plural: command_1.flags.string({ char: 'p', description: 'plural label for the UI' }),
    description: command_1.flags.string({ default: 'added from sfdx plugin', description: 'optional description so you can remember why you added this and what it\'s for' }),
    // type specific attributes
    enterprise: command_1.flags.boolean({ description: 'enable bulk/sharing/streaming' }),
    sharingmodel: command_1.flags.string({ description: 'sharing model', options: ['Read', 'ReadWrite', 'Private'], default: 'ReadWrite' }),
    activities: command_1.flags.boolean({ description: 'the enableActivities flag on an object (invalid for __b, __e)' }),
    search: command_1.flags.boolean({ description: 'the enableSearch flag on an object (invalid for __b, __e)' }),
    reports: command_1.flags.boolean({ description: 'the enableReports flag on an object (invalid for __b, __e)' }),
    history: command_1.flags.boolean({ description: 'the enableHistory flag on an object (invalid for __b, __e)' }),
    feeds: command_1.flags.boolean({ description: 'the enableFeeds flag on an object (invalid for __b, __e)' }),
    nametype: command_1.flags.string({ description: 'name field type', options: ['Text', 'AutoNumber'] }),
    namefieldlabel: command_1.flags.string({ description: 'the label for the name field', default: 'Name' }),
    autonumberformat: command_1.flags.string({ description: 'the display format for the autonumbering' }),
    highvolume: command_1.flags.boolean({ char: 'h', description: 'high volume, valid only for platform events (__e)' }),
    // general command params
    interactive: command_1.flags.boolean({ char: 'i', description: 'fully interactive--ask me every possible question.' }),
    directory: command_1.flags.directory({ char: 'd', default: 'force-app/main/default', description: 'where to create the folder (if it doesn\'t exist already) and file...defaults to force-app/main/default' })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
ObjectCreate.requiresProject = true;
exports.default = ObjectCreate;
//# sourceMappingURL=create.js.map