"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const fs = require("fs-extra");
const jsToXml = require("js2xmlparser");
const getExisting_1 = require("../../../shared/getExisting");
const options = require("../../../shared/js2xmlStandardOptions");
class FATUpdate extends command_1.SfdxCommand {
    async run() {
        const targetFolder = `${this.flags.directory}/objects/${this.flags.object}`;
        const targetFilename = `${targetFolder}/${this.flags.object}.object-meta.xml`;
        if (!fs.existsSync(targetFolder)) {
            this.ux.error(`Folder does not exist: ${targetFolder}.  Pull object schema using sfdx shane:mdapi:pull -o ${this.flags.object}`);
            return;
        }
        if (!fs.existsSync(targetFilename)) {
            this.ux.error(`Object does not exist in local source: ${targetFilename}.  Pull object schema using sfdx shane:mdapi:pull -s`);
            return;
        }
        let existing = await getExisting_1.getExisting(targetFilename, 'CustomObject');
        existing.enableHistory = true;
        existing.historyRetentionPolicy = existing.historyRetentionPolicy || {};
        if (this.flags.archiveaftermonths >= 0) {
            existing.historyRetentionPolicy.archiveAfterMonths = this.flags.archiveaftermonths;
        }
        if (this.flags.archiveretentionyears >= 0) {
            existing.historyRetentionPolicy.archiveRetentionYears = this.flags.archiveretentionyears;
        }
        if (this.flags.description) {
            existing.historyRetentionPolicy.description = this.flags.description;
        }
        existing = await getExisting_1.fixExistingDollarSign(existing);
        // convert to xml and write out the file
        const xml = jsToXml.parse('CustomObject', existing, options.js2xmlStandardOptions);
        fs.writeFileSync(targetFilename, xml);
        this.ux.log(chalk_1.default.green(`Updated ${targetFilename} in local source`));
        return existing;
    }
}
FATUpdate.description = 'add or update a field audit trail retention policy on an object.  Modifies local source--you still need to push/deploy';
FATUpdate.examples = [
    `sfdx shane:object:fat -o Account
// set the retention policy on Account to the defaults (archive after 18 months, archive for 10 years)
`,
    `sfdx shane:object:fat -o Account -m 4 -y 5
// archive history for 5 years, after being in regular history for 4 months
`,
    `sfdx shane:object:fat -o Account -m 4 -y 5 -d myDir
// same as 2nd example, except metadata is in myDir instead of the default force-app/main/default
`,
    `sfdx shane:mdapi:pull -o Account -u realOrg && sfdx shane:object:fat -o Account -m 4 -y 5 -d myDir && sfdx shane:mdapi:push -u realOrg
// get some object you don't have locally, create the policy, and push that back up to where it came from
`
];
FATUpdate.flagsConfig = {
    object: command_1.flags.string({ char: 'o', description: 'object to manage the policy for', required: true }),
    archiveaftermonths: command_1.flags.integer({ char: 'm', description: 'archive after this number of months', default: 18, min: 1, max: 18 }),
    archiveretentionyears: command_1.flags.integer({ char: 'y', default: 10, min: 0, max: 10, description: 'Archive for this many years' }),
    description: command_1.flags.string({ description: 'optional friendly description for the policy' }),
    directory: command_1.flags.directory({ char: 'd', default: 'force-app/main/default', description: 'Where is all this metadata? defaults to force-app/main/default' })
};
FATUpdate.requiresProject = true;
exports.default = FATUpdate;
//# sourceMappingURL=fat.js.map