"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const cli_ux_1 = require("cli-ux");
const fs = require("fs-extra");
const jsToXml = require("js2xmlparser");
const util = require("util");
const xml2js = require("xml2js");
const getExisting_1 = require("../../../shared/getExisting");
const options = require("../../../shared/js2xmlStandardOptions");
const SupportedTypes__b = ['Text', 'Number', 'DateTime', 'Lookup', 'LongTextArea'];
const SupportedTypes__e = ['Text', 'Number', 'DateTime', 'Date', 'LongTextArea', 'Checkbox'];
const SupportedTypes__c = ['Text', 'Number', 'DateTime', 'Date', 'LongTextArea', 'Checkbox', 'Url', 'Email', 'Phone'];
class FieldCreate extends command_1.SfdxCommand {
    async run() {
        if (!this.flags.object) {
            this.flags.object = await cli_ux_1.default.prompt('object API name?');
        }
        const objectMetaPath = `${this.flags.directory}/objects/${this.flags.object}/${this.flags.object}.object-meta.xml`;
        // does it exist?
        if (!fs.existsSync(objectMetaPath)) {
            this.ux.error(chalk_1.default.red(`object not found: ${objectMetaPath}`));
            return;
        }
        if (!this.flags.name) {
            this.flags.name = await cli_ux_1.default.prompt('Label for your new field?');
        }
        if (!this.flags.api) {
            this.flags.api = await cli_ux_1.default.prompt('API name for your new field?', { default: `${this.flags.name.replace(/ /g, '_')}__c` });
        }
        // be helpful
        if (!this.flags.api.endsWith('__c')) {
            this.flags.api = this.flags.api + '__c';
        }
        const fieldsFolderPath = `${this.flags.directory}/objects/${this.flags.object}/fields`;
        const fieldMetaPath = `${this.flags.directory}/objects/${this.flags.object}/fields/${this.flags.api}.field-meta.xml`;
        fs.ensureDirSync(fieldsFolderPath);
        if (fs.existsSync(fieldMetaPath)) {
            this.ux.error(chalk_1.default.red(`field already exists ${fieldMetaPath}`));
            return;
        }
        while (this.flags.object.endsWith('__b') && !SupportedTypes__b.includes(this.flags.type)) {
            this.flags.type = await cli_ux_1.default.prompt(`Type (${SupportedTypes__b.join(',')})?`, { default: 'Text' });
        }
        while (this.flags.object.endsWith('__e') && !SupportedTypes__e.includes(this.flags.type)) {
            this.flags.type = await cli_ux_1.default.prompt(`Type (${SupportedTypes__e.join(',')})?`, { default: 'Text' });
        }
        while (this.flags.object.endsWith('__c') && !SupportedTypes__c.includes(this.flags.type)) {
            this.flags.type = await cli_ux_1.default.prompt(`Type (${SupportedTypes__c.join(',')})?`, { default: 'Text' });
        }
        // we have at least these two fields now
        const outputJSON = {
            label: this.flags.name,
            type: this.flags.type,
            fullName: this.flags.api
        };
        // type specific values
        if (this.flags.type === 'Text') {
            if (this.flags.length >= 0) {
                outputJSON.length = this.flags.length;
            }
            else {
                outputJSON.length = await cli_ux_1.default.prompt('Length? (Max 255)', { default: '255' });
            }
        }
        if (this.flags.type === 'Checkbox') {
            outputJSON.defaultValue = this.flags.default || await cli_ux_1.default.prompt('Default value (required for checkboxes)?', { default: 'false' });
        }
        if (this.flags.type === 'LongTextArea') {
            if (this.flags.length >= 0) {
                outputJSON.length = this.flags.length;
            }
            else {
                outputJSON.length = await cli_ux_1.default.prompt('Length? (Max 131072)', { default: '131072' });
            }
            outputJSON.visibleLines = 3;
        }
        if (this.flags.type === 'Lookup') {
            outputJSON.referenceTo = this.flags.lookupobject || await cli_ux_1.default.prompt('What object for Lookup field? ex: Account, Something__c');
            outputJSON.relationshipName = this.flags.relname || await cli_ux_1.default.prompt('relationship api name?');
            outputJSON.relationshipLabel = outputJSON.relationshipName;
        }
        if (this.flags.type === 'Number') {
            if (this.flags.scale >= 0) {
                outputJSON.scale = this.flags.scale;
            }
            else {
                outputJSON.scale = await cli_ux_1.default.prompt('how many decimal places (scale)?', { default: '0' });
            }
            if (this.flags.precision >= 0) {
                outputJSON.precision = this.flags.precision;
            }
            else {
                outputJSON.precision = await cli_ux_1.default.prompt(`how many total digits, including those ${outputJSON.scale} decimal places? (precision, MAX ${18 - outputJSON.scale})?`, { default: `${18 - outputJSON.scale}` });
            }
        }
        // optional stuff
        if (this.flags.required) {
            outputJSON.required = true;
        }
        else if (this.flags.interactive && outputJSON.type !== 'Checkbox') {
            outputJSON.required = await cli_ux_1.default.confirm('required? (y/n)');
        }
        if (this.flags.unique) {
            outputJSON.unique = true;
        }
        else if (this.flags.interactive && ['Number', 'Text'].includes(outputJSON.type)) {
            outputJSON.unique = await cli_ux_1.default.confirm('unique? (y/n)');
        }
        if (this.flags.externalid) {
            outputJSON.externalId = true;
        }
        else if (this.flags.interactive && !this.flags.object.endsWith('__e') && ['Number', 'Text'].includes(outputJSON.type)) {
            outputJSON.externalId = await cli_ux_1.default.confirm('external ID?  (y/n)');
        }
        if (this.flags.description) {
            outputJSON.description = this.flags.description;
        }
        else if (this.flags.interactive) {
            outputJSON.description = await cli_ux_1.default.prompt('description?  Be nice to your future self!', { required: false });
        }
        if (this.flags.helptext) {
            outputJSON.inlineHelpText = this.flags.helptext;
        }
        else if (this.flags.interactive && !this.flags.object.endsWith('__e')) {
            outputJSON.inlineHelpText = await cli_ux_1.default.prompt('inline help text?  Be nice to your users!', { required: false, default: outputJSON.description });
        }
        if (this.flags.trackhistory) {
            outputJSON.trackHistory = this.flags.trackhistory;
        }
        else if (this.flags.interactive && this.flags.object.endsWith('__c')) {
            outputJSON.trackHistory = await cli_ux_1.default.confirm('enable history tracking?  (y/n)');
        }
        // dealing with big object indexes
        if (this.flags.object.includes('__b') && !this.flags.noindex) {
            const parser = new xml2js.Parser({ explicitArray: true });
            const parseString = util.promisify(parser.parseString);
            const filePath = `${this.flags.directory}/objects/${this.flags.object}/${this.flags.object}.object-meta.xml`;
            const fileRead = await parseString(fs.readFileSync(filePath));
            let existing = fileRead.CustomObject;
            // this.ux.log(existing.indexes[0].fields);
            existing.indexes[0].fields = existing.indexes[0].fields || [];
            // this.ux.log(existing.indexes[0].fields);
            while (!(this.flags.indexposition > -1) && !this.flags.indexappend && !this.flags.noindex) {
                const response = await cli_ux_1.default.prompt(`where in the big object index? Enter an array key (0 is first.  There are already ${existing.indexes[0].fields.length}) or the word LAST (add to the end) or NO (don't index this field)`, { default: 'LAST' });
                if (response === 'NONE') {
                    this.flags.noindex = true;
                    // this.flags.indexappend = true;
                }
                else if (response === 'LAST') {
                    this.flags.indexappend = true;
                }
                else {
                    if (this.flags.indexposition >= 0) {
                        this.flags.indexposition = response;
                    }
                }
            }
            if (this.flags.noindex) {
                return; // we're done.  Just quit!
            }
            // make it required since we're going to index it
            outputJSON.required = true;
            // we were told what to do
            while (this.flags.indexdirection !== 'ASC' && this.flags.indexdirection !== 'DESC') {
                outputJSON.required = true;
                this.flags.indexdirection = await cli_ux_1.default.prompt('which direction should this index be sorted? (ASC, DESC)', { default: 'DESC' });
            }
            existing = await getExisting_1.fixExistingDollarSign(existing);
            const newIndex = {
                name: this.flags.api,
                sortDirection: this.flags.indexdirection
            };
            if (this.flags.indexappend) {
                existing.indexes[0].fields.push(newIndex);
            }
            else {
                existing.indexes[0].fields.splice(this.flags.indexposition, 0, newIndex);
            }
            const position = this.flags.indexposition || existing.indexes[0].fields.length - 1;
            // conver to xml and write out the file
            const objXml = jsToXml.parse('CustomObject', existing, options.js2xmlStandardOptions);
            fs.writeFileSync(filePath, objXml);
            this.ux.log(chalk_1.default.green(`Index for ${this.flags.api} added as [${position}] of ${existing.indexes[0].fields.length}`));
        }
        // write out the field xml
        const xml = jsToXml.parse('CustomField', outputJSON, options.js2xmlStandardOptions);
        fs.writeFileSync(fieldMetaPath, xml);
        this.ux.log(`Created ${chalk_1.default.green(fieldMetaPath)}.  Add perms with ${chalk_1.default.cyan(`sfdx shane:permset:create -o ${this.flags.object} -f ${this.flags.api} -n yourPermSetName`)}`);
    }
}
FieldCreate.description = 'create or add fields to an existing object';
FieldCreate.examples = [
    `sfdx shane:object:field
// without any params, the cli is going to ask you questions to generate your field interactively
`,
    `sfdx shane:object:field --api My_Field__c -l 255 -n "My Field" -t Text -o  BigTest__b --noindex
// create new text field called My Field (My_Field__c) on BigObject BigTest__b
`,
    `sfdx shane:object:field --api My_Index_Field__c -l 255 -n "My Index Field" -t Text -o  BigTest__b --indexdirection ASC --indexposition 1
// create new text field called My Field (My_Field__c) on BigObject BigTest__b, add it to the existing index as the second field
`,
    `sfdx shane:object:field --api My_Field__c -l 255 -n "My Field" -t Text -o  EventTest__e
// create new text field called My Field (My_Field__c) on Platform Event EventTest__e
`
];
FieldCreate.flagsConfig = {
    // common flags for field types
    object: command_1.flags.string({ char: 'o', description: 'API name of an object to add a field to' }),
    name: command_1.flags.string({ char: 'n', description: 'Label for the field' }),
    api: command_1.flags.string({ char: 'a', description: 'API name for the field' }),
    type: command_1.flags.string({ char: 't', description: `field type.  Big Objects: ${SupportedTypes__b.join(',')}.  Events: ${SupportedTypes__e.join(',')}.  Regular Objects: ${SupportedTypes__c.join(',')}` }),
    description: command_1.flags.string({ description: 'optional description for the field so you remember what it\'s for next year' }),
    default: command_1.flags.string({ description: 'required for checkbox fields.  Express in Salesforce formula language (good luck with that!)' }),
    required: command_1.flags.boolean({ char: 'r', description: 'field is required' }),
    unique: command_1.flags.boolean({ char: 'u', description: 'field must be unique' }),
    externalid: command_1.flags.boolean({ description: 'use as an external id' }),
    trackhistory: command_1.flags.boolean({ description: 'enable history tracking on the field' }),
    helptext: command_1.flags.string({ description: 'optional inline help text' }),
    // type specific flags
    length: command_1.flags.integer({ char: 'l', description: 'length (for text fields and text area)' }),
    scale: command_1.flags.integer({ char: 's', description: 'places right of the decimal' }),
    precision: command_1.flags.integer({ description: 'maximum allowed digits of a number, including whole and decimal places' }),
    lookupobject: command_1.flags.string({ description: 'API name of the object the lookup goes to' }),
    relname: command_1.flags.string({ description: 'API name for the lookup relationship' }),
    // big object index handling
    indexposition: command_1.flags.integer({ description: 'put in a specific position in the big object index (0 is the first element).  You\'re responsible for dealing with producing a sane array' }),
    indexappend: command_1.flags.boolean({ description: 'put next in the big object index' }),
    indexdirection: command_1.flags.string({ description: 'sort direction for the big object index', options: ['ASC', 'DESC'] }),
    noindex: command_1.flags.boolean({ description: 'do not add this field to the index' }),
    // stuff about the command's behavior itself
    interactive: command_1.flags.boolean({ char: 'i', description: 'fully interactive--ask me every possible question.' }),
    directory: command_1.flags.directory({ char: 'd', default: 'force-app/main/default', description: 'Where is this object metadata? defaults to force-app/main/default' })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
FieldCreate.requiresProject = true;
exports.default = FieldCreate;
//# sourceMappingURL=field.js.map