"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const fs = require("fs-extra");
const jsToXml = require("js2xmlparser");
const getExisting_1 = require("../../../shared/getExisting");
const setupArray_1 = require("../../../shared/setupArray");
const options = require("../../../shared/js2xmlStandardOptions");
class UnPerm extends command_1.SfdxCommand {
    async run() {
        const profileDirectory = `${this.flags.directory}/profiles`;
        const permsetDirectory = `${this.flags.directory}/permissionsets`;
        if (!this.flags.specific) {
            // just do all of them
            const profiles = fs.readdirSync(profileDirectory);
            const permsets = fs.readdirSync(permsetDirectory);
            for (const p of profiles) {
                const targetFilename = `${profileDirectory}/${p}`;
                await this.removePerms(targetFilename, 'Profile');
            }
            for (const p of permsets) {
                const targetFilename = `${permsetDirectory}/${p}`;
                await this.removePerms(targetFilename, 'PermissionSet');
            }
        }
        else if (this.flags.specific) {
            // ok, what kind is it and does it exist?
            if (fs.existsSync(`${profileDirectory}/${this.flags.specific}`)) {
                await this.removePerms(this.flags.specific, 'Profile');
            }
            else if (fs.existsSync(`${permsetDirectory}/${this.flags.specific}`)) {
                await this.removePerms(this.flags.specific, 'PermissionSet');
            }
            else {
                throw new Error(`not found: ${this.flags.specific}`);
            }
        }
    }
    async removePerms(targetFilename, metadataType) {
        let existing = await getExisting_1.getExisting(targetFilename, metadataType);
        existing = setupArray_1.setupArray(existing, 'objectPermissions');
        existing = setupArray_1.setupArray(existing, 'fieldPermissions');
        existing = setupArray_1.setupArray(existing, 'layoutAssignments');
        existing = setupArray_1.setupArray(existing, 'recordTypes');
        existing = setupArray_1.setupArray(existing, 'tabSettings');
        // this.ux.logJson(existing.objectPermissions);
        // this.ux.logJson(existing.fieldPermissions);
        const objectBefore = existing.objectPermissions.length;
        const fieldBefore = existing.fieldPermissions.length;
        const layoutBefore = existing.layoutAssignments.length;
        const recordTypeBefore = existing.recordTypeVisibilities.length;
        existing.objectPermissions = existing.objectPermissions.filter(item => item.object !== this.flags.object);
        existing.fieldPermissions = existing.fieldPermissions.filter(item => !item.field.startsWith(`${this.flags.object}.`));
        existing.layoutAssignments = existing.layoutAssignments.filter(item => !item.layout.startsWith(`${this.flags.object}-`));
        existing.recordTypeVisibilities = existing.recordTypeVisibilities.filter(item => !item.recordType.startsWith(`${this.flags.object}.`));
        existing.tabSettings = existing.tabSettings.filter(item => item.tab !== this.flags.object && item.tab !== `standard-${this.flags.object}`);
        existing.tabVisibilities = existing.tabVisibilities.filter(item => item.tab !== this.flags.object && item.tab !== `standard-${this.flags.object}`);
        existing = await getExisting_1.fixExistingDollarSign(existing);
        const outputXML = jsToXml.parse(metadataType, existing, options.js2xmlStandardOptions);
        fs.writeFileSync(targetFilename, outputXML);
        this.ux.log(`removed ${objectBefore - existing.objectPermissions.length} objects, ${recordTypeBefore - existing.recordTypeVisibilities.length} recordTypes, ${layoutBefore - existing.layoutAssignments.length} layout, ${fieldBefore - existing.fieldPermissions.length} fields from ${this.flags.object} ${chalk_1.default.blue(targetFilename)}`);
    }
}
UnPerm.description = 'remove references to an object from profiles/permsets (all or a specific one)';
UnPerm.examples = [
    `sfdx shane:object:unperm -o OpportunitySplit
// go through all the profiles/permsets in force-app/main/default and remove the object, field, recordtypes and layout assignments (profile only) for the named object
`
];
UnPerm.flagsConfig = {
    object: command_1.flags.string({ char: 'o', required: true, description: 'remove all references to an object from profiles or permsets' }),
    directory: command_1.flags.directory({ char: 'd', default: 'force-app/main/default', description: 'Where is all this metadata? defaults to force-app/main/default' }),
    specific: command_1.flags.string({ char: 's', description: 'specify a profile or permset by name to only remove it from that one' })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
UnPerm.requiresProject = true;
exports.default = UnPerm;
//# sourceMappingURL=unperm.js.map