"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const child_process = require("child_process");
const request = require("request-promise-native");
const stripcolor = require("strip-color");
const util = require("util");
const exec = util.promisify(child_process.exec);
const usernameURL = 'https://unique-username-generator.herokuapp.com/unique';
class CreateOrg extends command_1.SfdxCommand {
    async run() {
        // gets the unique username
        const usernameResult = await request.post({
            uri: usernameURL,
            body: {
                prefix: this.flags.userprefix,
                domain: this.flags.userdomain
            },
            json: true
        });
        // required flags
        let command = `sfdx force:org:create --json -f ${this.flags.definitionfile} -d ${this.flags.durationdays} -w ${this.flags.wait || 20}`;
        // optional value 	flags without defaults
        if (this.flags.clientid) {
            command = command + ` -i ${this.flags.clientid}`;
        }
        if (this.flags.targetdevhubusername) {
            command = command + ` -v ${this.flags.targetdevhubusername}`;
        }
        if (this.flags.setalias) {
            command = command + ` -a ${this.flags.setalias}`;
        }
        // optional boolean
        if (this.flags.noancestors) {
            command = command + ' -c';
        }
        if (this.flags.nonamespace) {
            command = command + ' -n';
        }
        if (this.flags.setdefaultusername) {
            command = command + ' -s';
        }
        if (this.flags.setdefaultusername) {
            command = command + ` username=${usernameResult.message}`;
        }
        this.ux.log(`executing ${command}`);
        const response = await exec(command);
        if (response.stdout) {
            const success = JSON.parse(stripcolor(response.stdout));
            if (success.status === 0) {
                this.ux.log(chalk_1.default.green(`Org created with id ${success.result.orgId} and username ${success.result.username} `));
            }
            else {
                this.ux.log(chalk_1.default.red(response.stderr));
            }
            return success.result;
        }
        else {
            return JSON.parse(response.stdout);
        }
    }
}
CreateOrg.description = 'create an org with a friendly username.  wraps force:org:create';
CreateOrg.examples = [
    `sfdx shane:org:create --userprefix shane -o org.test
// creates an org from the default project config/project-scratch-def.json but with username shane[i]@org.test where i is a unique sequence number for that -u/-o combination
`,
    `sfdx shane:org:create --userprefix shane -o org.test -a sydneyBristow -d 30 -v myOtherHub -f config/thatOtherFile.json
// above, but with an alias, a longer duration, and not the default hub, and not the default config file
`
];
CreateOrg.flagsConfig = {
    userprefix: command_1.flags.string({
        required: true,
        description: 'first part of the generated username.  Example: \'shane\' produces shane1@demo.org, shane2@demo.org'
    }),
    userdomain: command_1.flags.string({
        char: 'o',
        required: true,
        description: 'last part of the generated username (after the @ sign).  Example: \'demo.org\' produces shane1@demo.org, shane2@demo.org'
    }),
    clientid: command_1.flags.string({
        char: 'i',
        description: 'connected app consumer key'
    }),
    definitionfile: command_1.flags.filepath({
        char: 'f',
        default: 'config/project-scratch-def.json',
        description: 'path to a scratch org definition file.  Default = config/project-scratch-def.json'
    }),
    setalias: command_1.flags.string({
        char: 'a',
        description: 'set an alias for for the created scratch org'
    }),
    durationdays: command_1.flags.integer({
        char: 'd',
        default: 7,
        min: 1,
        max: 30,
        description: 'duration of the scratch org (in days) (default:7, min:1, max:30)'
    }),
    wait: command_1.flags.integer({
        description: 'the streaming client socket timeout (in minutes) (default:20, min:2)',
        default: 20,
        min: 2
    }),
    noancestors: command_1.flags.boolean({
        char: 'c',
        description: 'do not include second-generation package ancestors in the scratch org'
    }),
    nonamespace: command_1.flags.boolean({
        char: 'n',
        description: 'creates the scratch org with no namespace'
    }),
    setdefaultusername: command_1.flags.boolean({
        char: 's',
        description: 'set the created org as the default username'
    })
    // targetdevhubusername: flags.boolean({description: 'username or alias for the dev hub org; overrides default dev hub org' })
};
CreateOrg.requiresProject = true;
exports.default = CreateOrg;
//# sourceMappingURL=create.js.map