"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const chalk_1 = require("chalk");
const child_process = require("child_process");
const cli_ux_1 = require("cli-ux");
const fs = require("fs-extra");
const stripcolor = require("strip-color");
const util = require("util");
const exec = util.promisify(child_process.exec);
const pkgVersionFileName = 'latestVersion.json';
class Bump extends command_1.SfdxCommand {
    async run() {
        if ((this.flags.major && this.flags.minor) || (!this.flags.major && !this.flags.minor)) {
            this.ux.error(chalk_1.default.red('You have to specify either --major or --minor but not both'));
            return;
        }
        const projectFile = await this.project.retrieveSfdxProjectJson(false);
        const project = JSON.parse(fs.readFileSync(projectFile.getPath(), 'UTF-8'));
        const targetDirIndex = project.packageDirectories.findIndex(i => {
            return i.path === this.flags.target;
        });
        if ((targetDirIndex < 0)) {
            this.ux.error(`found nothing in packageDirectories matching path ${this.flags.path}`);
        }
        const versionNumber = project.packageDirectories[targetDirIndex].versionNumber.split('.');
        if (this.flags.major) {
            versionNumber[0] = parseInt(versionNumber[0], 10) + 1;
            versionNumber[1] = 0;
            versionNumber[2] = 0;
        }
        else if (this.flags.minor) {
            versionNumber[1] = parseInt(versionNumber[1], 10) + 1;
            versionNumber[2] = 0;
        }
        else if (this.flags.patch) {
            versionNumber[2] = parseInt(versionNumber[2], 10) + 1;
        }
        project.packageDirectories[targetDirIndex].versionNumber = versionNumber.join('.');
        await fs.writeFile(projectFile.getPath(), JSON.stringify(project, null, 2));
        this.ux.log(chalk_1.default.green(`Updated sfdx-project.json for ${this.flags.target} to ${project.packageDirectories[targetDirIndex].versionNumber}`));
        // interface actualResultI {
        //   Package2VersionId: string;
        //   SubscriberPackageVersionId: string;
        //   Status: string;
        // }
        // do we need to generate the new version?
        if (this.flags.create || this.flags.release) {
            try {
                cli_ux_1.default.action.start('Creating package version (this\'ll take a while)');
                // createResult = <CreateResultI>await exec(`sfdx force:package2:version:create -d ${this.flags.target} -w 20 -v ${this.hubOrg.getUsername()} --json`);
                const createResult = await exec(`sfdx force:package:version:create -x -d ${this.flags.target} -w 20 -v ${this.hubOrg.getUsername()} --json`);
                cli_ux_1.default.action.stop();
                // this.ux.logJson(JSON.parse(createResult.stdout));
                const actualResult = JSON.parse(stripcolor(createResult.stdout)).result;
                this.ux.logJson(actualResult);
                await fs.writeFile(`${this.project.getPath()}/${pkgVersionFileName}`, JSON.stringify(actualResult, null, 2));
                this.ux.log(chalk_1.default.green(`Version Created with Id: ${actualResult.Package2VersionId}.  Details written to ${pkgVersionFileName}`));
                // now, are we publishing
                if (this.flags.release) {
                    cli_ux_1.default.action.start(`Releasing: sfdx force:package:version:promote -n -p ${actualResult.Package2VersionId} -v ${this.hubOrg.getUsername()} --json`);
                    await exec(`sfdx force:package:version:promote -n -p ${actualResult.Package2VersionId} -v ${this.hubOrg.getUsername()} --json`);
                    cli_ux_1.default.action.stop();
                    this.ux.log(chalk_1.default.green(`Version released. May take several minutes to become available to destination org.  Install with sfdx force:package:install -r -b 20 -w 20 -p ${actualResult.SubscriberPackageVersionId} -u destinationOrgAlias`));
                }
                else {
                    return JSON.parse(createResult.stdout);
                }
            }
            catch (err) {
                this.ux.error(chalk_1.default.red(JSON.parse(err.stderr).message));
                cli_ux_1.default.action.stop();
                return err;
            }
        }
    }
}
Bump.description = 'bump the major/minor version number in the packageDirectory';
Bump.examples = [
    `sfdx shane:package2:version:bump -m
// bump the minor version up by one (and set patch to 0)
`,
    `sfdx shane:package2:version:bump -M
// bump the major version up by one (and set minor/patch to 0)
`,
    `sfdx shane:package2:version:bump -p
// bump the patch version up by one
`,
    `sfdx shane:package2:version:bump -M -t myDir
// bump the major version up by one for a particular directory that's not the default
`,
    `sfdx shane:package2:version:bump --minor --create
// bump the minor version up by one and create a new package2version
`,
    `sfdx shane:package2:version:bump --minor --release
// bump the minor version up by one and create a new package2version, then set that as released
`
];
Bump.flagsConfig = {
    major: command_1.flags.boolean({ char: 'M', description: 'Bump the major version by 1, sets minor,build to 0', exclusive: ['minor', 'patch'] }),
    minor: command_1.flags.boolean({ char: 'm', description: 'Bump the minor version by 1', exclusive: ['major', 'patch'] }),
    patch: command_1.flags.boolean({ char: 'p', description: 'Bump the patch version by 1', exclusive: ['major', 'minor'] }),
    create: command_1.flags.boolean({ char: 'c', description: 'create a new packageVersion from the new versionNumber' }),
    release: command_1.flags.boolean({ char: 'r', description: 'set the newly version as released (out of Beta).  Implies create whether you flag it or not :)' }),
    target: command_1.flags.string({ char: 't', default: 'force-app', description: 'name of your package directory (defaults to force-app)' })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
Bump.requiresProject = true;
Bump.requiresDevhubUsername = true;
exports.default = Bump;
//# sourceMappingURL=bump.js.map