"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = require("fs-extra");
const jsToXml = require("js2xmlparser");
const getExisting_1 = require("../../../../shared/getExisting");
const options = require("../../../../shared/js2xmlStandardOptions");
const chalk_1 = require("chalk");
class TSPUsernameUpdate extends command_1.SfdxCommand {
    async run() {
        const targetFolder = `${this.flags.directory}/transactionSecurityPolicies`;
        const finalUsername = this.flags.newusername || this.org.getUsername();
        const output = [];
        if (!fs.existsSync(targetFolder)) {
            this.ux.error(`Folder does not exist: ${targetFolder}`);
            return;
        }
        const tsps = fs.readdirSync(targetFolder);
        this.ux.log(`Updating ${tsps.length} transaction security policies`);
        // loop through the TSPs
        for (const tsp of tsps) {
            let existing = await getExisting_1.getExisting(`${targetFolder}/${tsp}`, 'TransactionSecurityPolicy');
            existing.executionUser = finalUsername;
            if (Array.isArray(existing.action.notifications)) {
                existing.action.notifications.forEach((notification, key) => {
                    existing.action.notifications[key].user = finalUsername;
                });
            }
            else {
                existing.action.notifications.user = finalUsername;
            }
            existing = await getExisting_1.fixExistingDollarSign(existing);
            // convert to xml and write out the file
            const xml = jsToXml.parse('TransactionSecurityPolicy', existing, options.js2xmlStandardOptions);
            fs.writeFileSync(`${targetFolder}/${tsp}`, xml);
            this.ux.log(chalk_1.default.green(`Updated ${tsp}`));
            output.push(existing);
        }
        return output;
    }
}
TSPUsernameUpdate.description = 'change the username on all transaction security policies';
TSPUsernameUpdate.examples = [
    `sfdx shane:tsp:username:update -n newusername@example.com
// updates the username for executionUser and all notifications in all transaction security policies
`,
    `sfdx shane:tsp:username:create
// updates the username for executionUser and all notifications in all transaction security policies to the default org's username
`,
    `sfdx shane:tsp:username:create -u someAlias
// updates the username for executionUser and all notifications in all transaction security policies to the specified target org's username
`
];
TSPUsernameUpdate.flagsConfig = {
    newusername: command_1.flags.email({ char: 'n', description: 'manually specify the username, ignoring your default or any -u' }),
    directory: command_1.flags.directory({ char: 'd', default: 'force-app/main/default', description: 'Where is all this metadata? defaults to force-app/main/default' })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
TSPUsernameUpdate.requiresProject = true;
TSPUsernameUpdate.supportsUsername = true;
exports.default = TSPUsernameUpdate;
//# sourceMappingURL=update.js.map